/*  FFT2DAT - v.0.1.4 */
/*  Niels Bassler <bassler@vp7.dk> */
/*  10.Jul.2002 */
/*  Change log: */
/*  v.0.1.5 MONO version. Scale factor now as argument. Little speed improvement.*/ 
/*  v.0.1.4 Even faster code (skipped poor recalculation during normilization, uses temp file instead) */
/*  v.0.1.3 Added dual channel support, hash function and duration argument, code optimiztion */
/*  v.0.1.2 FIX: low mem-usage, very long files can now be generated. */
/*  v.0.1  */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define SAMPLE_RATE           44100

#define FFT_THRESHOLD         0.5

#define DURATION              20      /* default duration in seconds */
#define SCALE_FACTOR          100000  /* 100000 => 2000 muHerz --> 200 Herz */
#define BASE_FREQ             440     /* base freqency (just testing) */

#define HASH_MARKS            20000
#define MAX_ARRAY_SIZE        100000
#define OVERFLOW_PROTECTION   100000



int main(int argc, char *argv[])

{

  FILE *stream, *temp_stream, *outputstream;

  long int sample_number = 0;
  long int total_length = 0;

  int j=0, i=0;

  float a,b,q,t=0;

  float duration = DURATION;
  float scale_factor = SCALE_FACTOR;

  float temp_val =0;

  float freq[MAX_ARRAY_SIZE], ampl[MAX_ARRAY_SIZE];
  long int array_size=0;

  float hash;

  double sample;
  double sample_min=0, sample_max=0;

  char *temp_filename;

  
  if (argv[3] != NULL)
    duration = atof(argv[3]);

  if (argv[4] != NULL)
    scale_factor = atof(argv[4]);

  total_length = (long int) SAMPLE_RATE * duration;

  printf("  Duration: %.2f seconds         Sample rate: %d Hz\n", duration, SAMPLE_RATE);
  printf("  FFT_threshold: %f\n", FFT_THRESHOLD);
  printf("  Freq.(muHz) Amplitude\n"); 
  printf("  Total length: %.0f\n", (float) total_length);


  if (argv[1] == NULL) {
    printf("   Useage:  %s inputfile.dat outputfile.dat [duration in s [scale factor]]\n",
	     argv[0]);
    exit(1);
  }

  
  /* reading array */
  temp_val = 2 * M_PI * (0.000001 * scale_factor);
  stream = fopen(argv[1], "r");
  while ((fscanf(stream,"%f %f",&a,&b)) 
	 && (!feof(stream)))
    {
       if (b > FFT_THRESHOLD) {	
	 
	 freq[j] = a * temp_val;
	 ampl[j] = b;
	 j++;
	 if ( j > MAX_ARRAY_SIZE ) {
	   printf( " Error: input data file too large.\n");
	   exit(1);
	 }
	 printf("  %f %f\n",a,b);
       }
    }
  fclose(stream);
  
  array_size = j;
  printf("   Array size %d\n",j);

  /* First run to detect limits, without normalization */
  printf("   Detecting limits...\n");

  /* get a template file*/
  /* (No, I do not b***** care about the warnings!) */
  temp_filename = tmpnam(NULL);
  temp_stream = fopen(temp_filename, "w");

  if (temp_stream == NULL) {
    printf("   Error: cannot open temp file.\n");
    exit(1);
  }


  /* DUAL CHANNEL VERSION: */
  while (sample_number <  total_length) {
    /* clear sample value */
    sample = 0;
    t = (float) sample_number / SAMPLE_RATE;

    /* CHANNEL 1 */
    for (j=0; j < array_size; ++j) 
      {
	/* single increement, use all numbers */
	q = ampl[j] * sin( (float) freq[j] * t);
	sample += q; 
	if (sample > OVERFLOW_PROTECTION)
	  {
	    printf("OOOOPS: %f --- %f %f %f %f",(float) sample,a,b,t,q);
	    exit(1);
	  }
      }

    /* check and record limits */
    if (sample > sample_max)
      sample_max = sample;
    if (sample < sample_min)
      sample_min = sample;

    fprintf(temp_stream,"%f %.12f\n",t,sample); 

    /* the mono channel have now been written to the one temp file. */
    /* next sample */
    ++i;

    if (i > HASH_MARKS) {
      hash =  ( (float) sample_number) / total_length * 100;
      printf("      %.1f %%\n", hash);
      i = 0;
    }   
    sample_number++;
  }

  fclose(temp_stream);



  printf("   MAX/MIN: %f %f\n", sample_max,sample_min);
  if (fabs(sample_min) > fabs(sample_max))
    sample_max = sample_min;


  /* ------------------- */
  /*   now normalize it: */
  /* --------------------*/

  printf("  %s",temp_filename);
  printf("  Normalizing ... (this may take some time)\n");

  temp_stream = fopen(temp_filename, "r");
  outputstream = fopen (argv[2], "w");
  fprintf(outputstream, "; Sample Rate %i\n", SAMPLE_RATE);


  while (!feof(temp_stream)) {
    /* CHANNEL 1 */
    fscanf(temp_stream,"%f %f", &t, &b);
    /* Why is this neccessary ? (Very spooky!)*/ 
    sample = b;
    /*        printf("1.  %f %f\n",t,sample);*/
    sample /= sample_max;
    fprintf(outputstream,"%f %.12f\n",t,sample);  
    
  }

  fclose(temp_stream);
  fclose(outputstream);

  remove(temp_filename);

  printf("   Done.\n");
  return(0);

}
